/******************************************************************************
 * igmpshow.c - An IGMP Proxy show utility
 ******************************************************************************
 * Name       : igmpshow.c
 * Description: Imlements IGMP proxy show routines
 * Date       : May 30, 2007
 * Authors    : sachink@student.utdallas.edu
 * Last Modif : May 30, 2007
 *
 *****************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "conf.h"

char* pidfile = DEFAULT_PID_FILE_NAME;
int main(int argc, char **argv)
{
	int fd, fdout, bytes, len=0, i;
	char buf[2048], *pbuf = buf;
	FILE *fp = fopen(pidfile, "r");

	/* PID lock file exists */
	if (fp) {
	  char pid_string[11];
	  if (fgets(pid_string, sizeof(pid_string), fp) != NULL) {
	    int oldpid;
	    oldpid = (int) strtol(pid_string, (char **)NULL, 10);
	    if (oldpid) {
	      if (kill(oldpid, 0) == 0) {
	        /* proxy process already running */
	      } else {
	        /* proxy process not running */
	        fprintf(stderr, "IGMP proxy not up\n");
	        return -1;
	      }
	    }
	  }
	} else {
	  fprintf(stderr, "IGMP proxy not up\n");
	  return -1; 
	}	
	fclose(fp);

	if(argc < 2) {
	  fprintf(stderr,"Usage: %s a | g <mcast address> | s <unicast address>"
	  		  "| h\n", argv[0]);
	  fprintf(stderr, "\t a: Prints IGMP table\n");
	  fprintf(stderr, "\t g: Prints mcast group information\n");
	  fprintf(stderr, "\t s: Prints mcast source information\n");
	  fprintf(stderr, "\t h: Prints help information\n");
	  return -1;
	}

	if(access(IGMP_PROXY_DEBUG_FIFO, F_OK) != 0 || 
	   access(IGMP_PROXY_DEBUG_FIFO_OUT, F_OK) != 0) {
	  fprintf(stderr, "IGMP proxy not running\n");
	  return -1;
	}

	fd = open(IGMP_PROXY_DEBUG_FIFO, O_RDWR);
	if (fd < 0) {
	  fprintf(stderr, "Could not open write fifo to igmp proxy\n");
	  return -1;
	}

	fdout = open(IGMP_PROXY_DEBUG_FIFO_OUT, O_RDWR);
	if (fd < 0) {
	  fprintf(stderr, "Could not open read fifo to igmp proxy\n");
	  return -1;
	}

	i = 1;
	while(i < argc) {
	  bytes = sprintf(pbuf, "%s ", argv[i++]);
	  pbuf += bytes;
	}
	write(fd, buf, 20);

	len = read(fdout, buf, MAX_MSGBUFSIZE);
	buf[len-1] = '\0';
	fprintf(stderr, "%s\n", buf);
	return 0;
}
